      subroutine bndytrans
c
c=======================================================================
c                                                                    ===
c  Replace the Provided Orlanski (PO) corrections to barotropic      ===
c  velocity with values to balance the divergenc of transport        ===
c  against the change in surface elevation.                          ===
c                                                                    ===
c  ------                                                            ===
c  Input:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /BNDATA/                                                          ===
c                                                                    ===
c     DETATBY   Surface elevation PO corrections.              (real)===
c     PBC0      Surface pressure boundary/near boundary values.(real)===
c                                                                    ===
c  /ONEDIM/                                                          ===
c                                                                    ===
c     CS    cosine metric factors at velocity points.       (real)   ===
c     CST   cosine metric factors at tracer points.         (real)   ===
c     DXU   zonal grid spacing across velocity boxes.       (real)   ===
c     DYU   meridional grid spacing across velocity boxes.  (real)   ===
c                                                                    ===
c  /SCALAR/                                                          ===
c                                                                    ===
c     GRAV     acceleration due to gravity.      (real; cm/s^2)      ===
c     C2DTSF   twice the length of timestep.     (real; s)           ===
c                                                                    ===
#if defined ext_tide & !defined mixtideonly
c  /TIDESP/                                                          ===
c                                                                    ===
c     SRFTD    tidal elevation.                         (real; cm)   ===
c                                                                    ===
#endif
c  /VERTICAL/                                                        ===
c                                                                    ===
c     ETAT    Surface elevation at tracer points.       (real; cm)   ===
c                                                                    ===
c  /VERTSLABS/                                                       ===
c                                                                    ===
c     HD      bottom topography at the tracer points.    (real)      ===
c     HDVCF   bottom topography at the velocity points               ===
c             with respect to mean free surface.         (real)      ===
c                                                                    ===
c  -------                                                           ===
c  Output:                                                           ===
c  -------                                                           ===
c                                                                    ===
c  Common Blocks:                                                    ===
c                                                                    ===
c  /BNDATA/                                                          ===
c                                                                    ===
c     DUBARO   Zonal zero-transport PO corrections.       (real)     ===
c     DVBARO   Meridional zero-transport PO corrections.  (real)     ===
c                                                                    ===
c  ------                                                            ===
c  Calls:                                                            ===
c  ------                                                            ===
c                                                                    ===
c  none                                                              ===
c                                                                    ===
c=======================================================================
c
c-----------------------------------------------------------------------
c  Define global data.
c-----------------------------------------------------------------------
c
#include <cdefs.h>
#include <param.h>
#include <pconst.h>
#include <bndata.h>
#include <onedim.h>
#include <scalar.h>
#if defined ext_tide & !defined mixtideonly
# include <tidesp.h>
#endif
#include <vertical.h>
#include <vertslabs.h>
c
c-----------------------------------------------------------------------
c  Define local data.
c-----------------------------------------------------------------------
c
      integer i
#ifndef cyclic
     &       ,j
#endif
      FLOAT
     &     detaa,detab,hside
#ifndef cyclic
     &     ,abgrdeta,detane,detanw,detase,detasw,detax,detay,wtU,wtV
#endif
c
c=======================================================================
c  Begin executable code.
c=======================================================================
c
c-----------------------------------------------------------------------
c  Compute flux through south wall.
c-----------------------------------------------------------------------
c
      do 10 i = 1, imu
         if (hdvcf(i,1).gt.c0) then
c
c  Compute perturbations to elevation at velocity point.
c
            detaa = (detatby(i,1,itbp,south)+detatby(i+1,1,itbp,south)+
     &            detatby(i,2,itbp,south)+detatby(i+1,2,itbp,south))*p25
c
            detab = (detatby(i,1,itbm,south)+detatby(i+1,1,itbm,south)+
     &            detatby(i,2,itbm,south)+detatby(i+1,2,itbm,south))*p25
c
c  Compute height of open boundary.
c
            hside = (hd(i,1)+etat(i,1)+hd(i+1,1)+etat(i+1,1))*cst(1)*p5
c
c  Balance change of height with flux through outer boundary.
c
            dvbaro(i,itbp,south) = (detab-detaa)*cs(1)*dyu(1)
     &                             /(hside*c2dtsf)
#ifdef transbc
            dvbaro(i,itbp,south) = dvbaro(i,itbp,south)*hdvcf(i,1)
#endif
          else
c
c  Block flow through closed boundary.
c
            dvbaro(i,itbp,south) = c0
         endif
  10  continue
c
c-----------------------------------------------------------------------
c  Compute flux through north wall.
c-----------------------------------------------------------------------
c
      do 20 i = 1, imu
         if (hdvcf(i,jmtm1).gt.c0) then
c
c  Compute perturbations to elevation at velocity point.
c
            detaa = (detatby(i,1,itbp,north)+detatby(i+1,1,itbp,north)+
     &            detatby(i,2,itbp,north)+detatby(i+1,2,itbp,north))*p25
c
            detab = (detatby(i,1,itbm,north)+detatby(i+1,1,itbm,north)+
     &            detatby(i,2,itbm,north)+detatby(i+1,2,itbm,north))*p25
c
c  Compute height of open boundary.
c
            hside = (hd(i,jmt)+etat(i,jmt)+hd(i+1,jmt)+etat(i+1,jmt))
     &              *cst(jmt)*p5
c
c  Balance change of height with flux through outer boundary.
c
            dvbaro(i,itbp,north) = (detaa-detab)*cs(jmtm1)*dyu(jmtm1)
     &                             /(hside*c2dtsf)
#ifdef transbc
            dvbaro(i,itbp,north) = dvbaro(i,itbp,north)*hdvcf(i,jmtm1)
#endif
          else
c
c  Block flow through closed boundary.
c
            dvbaro(i,itbp,north) = c0
         endif
  20  continue
c
#ifndef cyclic
c-----------------------------------------------------------------------
c  Compute flux through west wall.
c-----------------------------------------------------------------------
c
      do 30 j = 1, jmtm1
         if (hdvcf(1,j).gt.c0) then
c
c  Compute perturbations to elevation at velocity point.
c
            detaa = (detatby(j,1,itbp,west)+detatby(j+1,1,itbp,west)+
     &              detatby(j,2,itbp,west)+detatby(j+1,2,itbp,west))*p25
c
            detab = (detatby(j,1,itbm,west)+detatby(j+1,1,itbm,west)+
     &              detatby(j,2,itbm,west)+detatby(j+1,2,itbm,west))*p25
c
c  Compute height of open boundary.
c
            hside = (hd(1,j)+etat(1,j)+hd(1,j+1)+etat(1,j+1))*p5
c
c  Balance change of height with flux through outer boundary.
c
            dubaro(j,itbp,west) = (detab-detaa)*cs(j)*dxu(1)
     &                             /(hside*c2dtsf)
# ifdef transbc
            dubaro(j,itbp,west) = dubaro(j,itbp,west)*hdvcf(1,j)
# endif
          else
c
c  Block flow through closed boundary.
c
            dubaro(j,itbp,west) = c0
         endif
  30  continue
c
c-----------------------------------------------------------------------
c  Compute flux through east wall.
c-----------------------------------------------------------------------
c
      do 40 j = 1, jmtm1
         if (hdvcf(imu,j).gt.c0) then
c
c  Compute perturbations to elevation at velocity point.
c
            detaa = (detatby(j,1,itbp,east)+detatby(j+1,1,itbp,east)+
     &              detatby(j,2,itbp,east)+detatby(j+1,2,itbp,east))*p25
c
            detab = (detatby(j,1,itbm,east)+detatby(j+1,1,itbm,east)+
     &              detatby(j,2,itbm,east)+detatby(j+1,2,itbm,east))*p25
c
c  Compute height of open boundary.
c
            hside = (hd(imu,j)+etat(imu,j)+hd(imu,j+1)+etat(imu,j+1))*p5
c
c  Balance change of height with flux through outer boundary.
c
            dubaro(j,itbp,east) = (detaa-detab)*cs(j)*dxu(imu)
     &                             /(hside*c2dtsf)
# ifdef transbc
            dubaro(j,itbp,east) = dubaro(j,itbp,east)*hdvcf(imu,j)
# endif
          else
c
c  Block flow through closed boundary.
c
            dubaro(j,itbp,east) = c0
         endif
  40  continue
c
c-----------------------------------------------------------------------
c  Have "double dipped" on corners, need to rescale flows.
c-----------------------------------------------------------------------
c
c SW corner
c
      detanw = detatby(2,1,itbp,west) - detatby(2,1,itbm,west)
      detane = detatby(2,2,itbp,west) - detatby(2,2,itbm,west)
      detasw = detatby(1,1,itbp,west) - detatby(1,1,itbm,west)
      detase = detatby(1,2,itbp,west) - detatby(1,2,itbm,west)
c
      detax    = p5*abs((detane-detanw) + (detase-detasw))
      detay    = p5*abs((detane-detase) + (detanw-detasw))
      abgrdeta = detax + detay
c
      if (abgrdeta.gt.c0) then
         wtU = detax/abgrdeta
       else
         wtU = p5
      endif
      wtV = c1 - wtU
c
      dubaro(1,itbp,west)  = wtU*dubaro(1,itbp,west)
      dvbaro(1,itbp,south) = wtV*dvbaro(1,itbp,south)
c
c SE corner
c
      detanw = detatby(2,2,itbp,east) - detatby(2,2,itbm,east)
      detane = detatby(2,1,itbp,east) - detatby(2,1,itbm,east)
      detasw = detatby(1,2,itbp,east) - detatby(1,2,itbm,east)
      detase = detatby(1,1,itbp,east) - detatby(1,1,itbm,east)
c
      detax    = p5*abs((detane-detanw) + (detase-detasw))
      detay    = p5*abs((detane-detase) + (detanw-detasw))
      abgrdeta = detax + detay
c
      if (abgrdeta.gt.c0) then
         wtU = detax/abgrdeta
       else
         wtU = p5
      endif
      wtV = c1 - wtU
c
      dubaro(1,itbp,east)  = wtU*dubaro(1,itbp,east)
      dvbaro(1,itbp,south) = wtV*dvbaro(1,itbp,south)
c
c NW corner
c
      detanw = detatby(jmt,1,itbp,west)   - detatby(jmt,1,itbm,west)
      detane = detatby(jmt,2,itbp,west)   - detatby(jmt,2,itbm,west)
      detasw = detatby(jmtm1,1,itbp,west) - detatby(jmtm1,1,itbm,west)
      detase = detatby(jmtm1,2,itbp,west) - detatby(jmtm1,2,itbm,west)
c
      detax    = p5*abs((detane-detanw) + (detase-detasw))
      detay    = p5*abs((detane-detase) + (detanw-detasw))
      abgrdeta = detax + detay
c
      if (abgrdeta.gt.c0) then
         wtU = detax/abgrdeta
       else
         wtU = p5
      endif
      wtV = c1 - wtU
c
      dubaro(jmtm1,itbp,west) = wtU*dubaro(jmtm1,itbp,west)
      dvbaro(1,itbp,north)    = wtV*dvbaro(1,itbp,north)
c
c NE corner
c
      detanw = detatby(jmt,2,itbp,east)   - detatby(jmt,2,itbm,east)
      detane = detatby(jmt,1,itbp,east)   - detatby(jmt,1,itbm,east)
      detasw = detatby(jmtm1,2,itbp,east) - detatby(jmtm1,2,itbm,east)
      detase = detatby(jmtm1,1,itbp,east) - detatby(jmtm1,1,itbm,east)
c
      detax    = p5*abs((detane-detanw) + (detase-detasw))
      detay    = p5*abs((detane-detase) + (detanw-detasw))
      abgrdeta = detax + detay
c
      if (abgrdeta.gt.c0) then
         wtU = detax/abgrdeta
       else
         wtU = p5
      endif
      wtV = c1 - wtU
c
      dubaro(jmtm1,itbp,east)  = wtU*dubaro(jmtm1,itbp,east)
      dvbaro(imtm1,itbp,north) = wtV*dvbaro(imtm1,itbp,north)
c
#endif
      return
      end
